@@ -1,6 +1,9 @@
 #[cfg(feature = "pyo3")]
 use std::hash::{Hash, Hasher};
-use std::{cmp::Ordering, str::FromStr};
+
+use std::cmp::Ordering;
+use std::ops::Bound;
+use std::str::FromStr;
 
 #[cfg(feature = "pyo3")]
 use pyo3::{
@@ -250,7 +253,18 @@ impl std::error::Error for VersionSpecifiersParseError {}
 /// let version_specifier = VersionSpecifier::from_str("== 1.*").unwrap();
 /// assert!(version_specifier.contains(&version));
 /// ```
-#[derive(Eq, PartialEq, Debug, Clone, Hash, rkyv::Archive, rkyv::Deserialize, rkyv::Serialize)]
+#[derive(
+    Eq,
+    Ord,
+    PartialEq,
+    PartialOrd,
+    Debug,
+    Clone,
+    Hash,
+    rkyv::Archive,
+    rkyv::Deserialize,
+    rkyv::Serialize,
+)]
 #[archive(check_bytes)]
 #[archive_attr(derive(Debug))]
 #[cfg_attr(feature = "pyo3", pyclass(get_all))]
@@ -381,6 +395,14 @@ impl VersionSpecifier {
         }
     }
 
+    /// `>=<version>`
+    pub fn greater_than_equal_version(version: Version) -> Self {
+        Self {
+            operator: Operator::GreaterThanEqual,
+            version,
+        }
+    }
+
     /// Get the operator, e.g. `>=` in `>= 2.0.0`
     pub fn operator(&self) -> &Operator {
         &self.operator
@@ -396,6 +418,50 @@ impl VersionSpecifier {
         self.version.any_prerelease()
     }
 
+    /// Returns the version specifiers whose union represents the given range.
+    pub fn from_bounds(
+        bounds: (&Bound<Version>, &Bound<Version>),
+    ) -> impl Iterator<Item = VersionSpecifier> {
+        let (b1, b2) = match bounds {
+            (Bound::Included(v1), Bound::Included(v2)) if v1 == v2 => {
+                (Some(VersionSpecifier::equals_version(v1.clone())), None)
+            }
+            (lower, upper) => (
+                VersionSpecifier::from_lower_bound(lower),
+                VersionSpecifier::from_upper_bound(upper),
+            ),
+        };
+
+        b1.into_iter().chain(b2)
+    }
+
+    /// Returns a version specifier representing the given lower bound.
+    fn from_lower_bound(bound: &Bound<Version>) -> Option<VersionSpecifier> {
+        match bound {
+            Bound::Included(version) => Some(
+                VersionSpecifier::from_version(Operator::GreaterThanEqual, version.clone())
+                    .unwrap(),
+            ),
+            Bound::Excluded(version) => Some(
+                VersionSpecifier::from_version(Operator::GreaterThan, version.clone()).unwrap(),
+            ),
+            Bound::Unbounded => None,
+        }
+    }
+
+    /// Returns a version specifier representing the given upper bound.
+    fn from_upper_bound(bound: &Bound<Version>) -> Option<VersionSpecifier> {
+        match bound {
+            Bound::Included(version) => Some(
+                VersionSpecifier::from_version(Operator::LessThanEqual, version.clone()).unwrap(),
+            ),
+            Bound::Excluded(version) => {
+                Some(VersionSpecifier::from_version(Operator::LessThan, version.clone()).unwrap())
+            }
+            Bound::Unbounded => None,
+        }
+    }
+
     /// Whether the given version satisfies the version range
     ///
     /// e.g. `>=1.19,<2.0` and `1.21` -> true
