@@ -47,6 +47,7 @@ use crate::pubgrub::{
     PubGrubPriorities, PubGrubPython, PubGrubSpecifier,
 };
 use crate::python_requirement::PythonRequirement;
+use crate::requires_python::RequiresPython;
 use crate::resolution::ResolutionGraph;
 pub(crate) use crate::resolver::availability::{
     IncompletePackage, ResolverVersion, UnavailablePackage, UnavailableReason, UnavailableVersion,
@@ -94,6 +95,14 @@ struct ResolverState<InstalledPackages: InstalledPackagesProvider> {
     /// When not set, the resolver is in "universal" mode.
     markers: Option<MarkerEnvironment>,
     python_requirement: PythonRequirement,
+    /// This is derived from `PythonRequirement` once at initialization
+    /// time. It's used in universal mode to filter our dependencies with
+    /// a `python_version` marker expression that has no overlap with the
+    /// `Requires-Python` specifier.
+    ///
+    /// This is non-None if and only if the resolver is operating in
+    /// universal mode. (i.e., when `markers` is `None`.)
+    requires_python: Option<MarkerTree>,
     selector: CandidateSelector,
     index: InMemoryIndex,
     installed_packages: InstalledPackages,
@@ -147,8 +156,7 @@ impl<'a, Context: BuildContext, InstalledPackages: InstalledPackagesProvider>
             AllowedYanks::from_manifest(&manifest, markers, options.dependency_mode),
             hasher,
             options.exclude_newer,
-            build_context.no_binary(),
-            build_context.no_build(),
+            build_context.build_options(),
         );
 
         Self::new_custom_io(
@@ -181,6 +189,16 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
         provider: Provider,
         installed_packages: InstalledPackages,
     ) -> Result<Self, ResolveError> {
+        let requires_python = if markers.is_some() {
+            None
+        } else {
+            Some(
+                python_requirement
+                    .requires_python()
+                    .map(RequiresPython::to_marker_tree)
+                    .unwrap_or_else(|| MarkerTree::And(vec![])),
+            )
+        };
         let state = ResolverState {
             index: index.clone(),
             git: git.clone(),
@@ -200,6 +218,7 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
             hasher: hasher.clone(),
             markers: markers.cloned(),
             python_requirement: python_requirement.clone(),
+            requires_python,
             reporter: None,
             installed_packages,
         };
@@ -306,6 +325,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             pins: FilePins::default(),
             priorities: PubGrubPriorities::default(),
             added_dependencies: FxHashMap::default(),
+            markers: MarkerTree::And(vec![]),
         };
         let mut forked_states = vec![state];
         let mut resolutions = vec![];
@@ -479,6 +499,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     let forked_deps = self.get_dependencies_forking(
                         &package,
                         &version,
+                        &state.markers,
                         &mut state.priorities,
                         &request_sink,
                     )?;
@@ -553,6 +574,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                                 if !is_last {
                                     cur_state = Some(forked_state.clone());
                                 }
+                                forked_state.markers.and(fork.markers);
 
                                 // Add that package and version if the dependencies are not problematic.
                                 let dep_incompats =
@@ -922,10 +944,11 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         &self,
         package: &PubGrubPackage,
         version: &Version,
+        markers: &MarkerTree,
         priorities: &mut PubGrubPriorities,
         request_sink: &Sender<Request>,
     ) -> Result<ForkedDependencies, ResolveError> {
-        let result = self.get_dependencies(package, version, priorities, request_sink);
+        let result = self.get_dependencies(package, version, markers, priorities, request_sink);
         if self.markers.is_some() {
             return result.map(|deps| match deps {
                 Dependencies::Available(deps) => ForkedDependencies::Unforked(deps),
@@ -941,6 +964,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         &self,
         package: &PubGrubPackage,
         version: &Version,
+        markers: &MarkerTree,
         priorities: &mut PubGrubPriorities,
         request_sink: &Sender<Request>,
     ) -> Result<Dependencies, ResolveError> {
@@ -959,6 +983,8 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     &self.locals,
                     &self.git,
                     self.markers.as_ref(),
+                    self.requires_python.as_ref(),
+                    markers,
                 );
 
                 let dependencies = match dependencies {
@@ -1109,6 +1135,8 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     &self.locals,
                     &self.git,
                     self.markers.as_ref(),
+                    self.requires_python.as_ref(),
+                    markers,
                 )?;
 
                 for (dep_package, dep_version) in dependencies.iter() {
@@ -1516,6 +1544,21 @@ struct SolveState {
     /// This keeps track of the set of versions for each package that we've
     /// already visited during resolution. This avoids doing redundant work.
     added_dependencies: FxHashMap<PubGrubPackage, FxHashSet<Version>>,
+    /// The marker expression that created this state.
+    ///
+    /// The root state always corresponds to a marker expression that is always
+    /// `true` for every `MarkerEnvironment`.
+    ///
+    /// In non-universal mode, forking never occurs and so this marker
+    /// expression is always `true`.
+    ///
+    /// Whenever dependencies are fetched, all requirement specifications
+    /// are checked for disjointness with the marker expression of the fork
+    /// in which those dependencies were fetched. If a requirement has a
+    /// completely disjoint marker expression (i.e., it can never be true given
+    /// that the marker expression that provoked the fork is true), then that
+    /// dependency is completely ignored.
+    markers: MarkerTree,
 }
 
 impl SolveState {
@@ -1833,17 +1876,30 @@ enum Response {
     },
 }
 
-/// An enum used by [`DependencyProvider`] that holds information about package dependencies.
-/// For each [Package] there is a set of versions allowed as a dependency.
+/// Information about the dependencies for a particular package.
+///
+/// This effectively distills the dependency metadata of a package down into
+/// its pubgrub specific constituent parts: each dependency package has a range
+/// of possible versions.
 #[derive(Clone)]
 enum Dependencies {
     /// Package dependencies are not available.
     Unavailable(UnavailableVersion),
     /// Container for all available package versions.
+    ///
+    /// Note that in universal mode, it is possible and allowed for multiple
+    /// `PubGrubPackage` values in this list to have the same package name.
+    /// These conflicts are resolved via `Dependencies::fork`.
     Available(Vec<(PubGrubPackage, Range<Version>)>),
 }
 
 impl Dependencies {
+    /// Turn this flat list of dependencies into a potential set of forked
+    /// groups of dependencies.
+    ///
+    /// A fork *only* occurs when there are multiple dependencies with the same
+    /// name *and* those dependency specifications have corresponding marker
+    /// expressions that are completely disjoint with one another.
     fn fork(self) -> ForkedDependencies {
         use std::collections::hash_map::Entry;
 
@@ -1922,7 +1978,6 @@ impl Dependencies {
             // is look for a group in which there is some overlap. If so, this
             // package gets added to that fork group. Otherwise, we create a
             // new fork group.
-            // possible_forks.push(PossibleFork { packages: vec![] });
             let Some(possible_fork) = possible_forks.find_overlapping_fork_group(marker) else {
                 // Create a new fork since there was no overlap.
                 possible_forks.forks.push(PossibleFork {
@@ -1939,6 +1994,7 @@ impl Dependencies {
         }
         let mut forks = vec![Fork {
             dependencies: vec![],
+            markers: MarkerTree::And(vec![]),
         }];
         for (_, possible_forks) in by_name {
             let fork_groups = match possible_forks {
@@ -1957,9 +2013,16 @@ impl Dependencies {
             let mut new_forks: Vec<Fork> = vec![];
             for group in fork_groups.forks {
                 let mut new_forks_for_group = forks.clone();
-                for (index, _) in group.packages {
+                for (index, markers) in group.packages {
                     for fork in &mut new_forks_for_group {
                         fork.dependencies.push(deps[index].clone());
+                        // Each marker expression in a single fork is,
+                        // by construction, overlapping with at least
+                        // one other marker expression in this fork.
+                        // However, the symmetric differences may be
+                        // non-empty. Therefore, the markers need to be
+                        // combined on the corresponding fork
+                        fork.markers.and(markers.clone());
                     }
                 }
                 new_forks.extend(new_forks_for_group);
@@ -1970,11 +2033,19 @@ impl Dependencies {
     }
 }
 
+/// Information about the (possibly forked) dependencies for a particular
+/// package.
+///
+/// This is like `Dependencies` but with an extra variant that only occurs when
+/// a `Dependencies` list has multiple dependency specifications with the same
+/// name and non-overlapping marker expressions (i.e., a fork occurs).
 #[derive(Debug)]
 enum ForkedDependencies {
     /// Package dependencies are not available.
     Unavailable(UnavailableVersion),
     /// No forking occurred.
+    ///
+    /// This is the same as `Dependencies::Available`.
     Unforked(Vec<(PubGrubPackage, Range<Version>)>),
     /// Forked containers for all available package versions.
     ///
@@ -1984,14 +2055,56 @@ enum ForkedDependencies {
     Forked(Vec<Fork>),
 }
 
+/// A single fork in a list of dependencies.
+///
+/// A fork corresponds to the full list of dependencies for a package,
+/// but with any conflicting dependency specifications omitted. For
+/// example, if we have `a<2 ; sys_platform == 'foo'` and `a>=2 ;
+/// sys_platform == 'bar'`, then because the dependency specifications
+/// have the same name and because the marker expressions are disjoint,
+/// a fork occurs. One fork will contain `a<2` but not `a>=2`, while
+/// the other fork will contain `a>=2` but not `a<2`.
 #[derive(Clone, Debug)]
 struct Fork {
+    /// The list of dependencies for this fork, guaranteed to be conflict
+    /// free. (i.e., There are no two packages with the same name with
+    /// non-overlapping marker expressions.)
     dependencies: Vec<(PubGrubPackage, Range<Version>)>,
+    /// The markers that provoked this fork.
+    ///
+    /// So in the example above, the `a<2` fork would have
+    /// `sys_platform == 'foo'`, while the `a>=2` fork would have
+    /// `sys_platform == 'bar'`.
+    ///
+    /// (This doesn't include any marker expressions from a parent fork.)
+    markers: MarkerTree,
 }
 
+/// Intermediate state that represents a *possible* grouping of forks
+/// for one package name.
+///
+/// This accumulates state while examining a `Dependencies` list. In
+/// particular, it accumulates conflicting dependency specifications and marker
+/// expressions. As soon as a fork can be ruled out, this state is switched to
+/// `NoForkPossible`. If, at the end of visiting all `Dependencies`, we still
+/// have `PossibleForks::PossiblyForking`, then a fork exists if and only if
+/// one of its groups has length bigger than `1`.
+///
+/// One common way for a fork to be known to be impossible is if there exists
+/// conflicting dependency specifications where at least one is unconditional.
+/// For example, `a<2` and `a>=2 ; sys_platform == 'foo'`. In this case, `a<2`
+/// has a marker expression that is always true and thus never disjoint with
+/// any other marker expression. Therefore, there can be no fork for `a`.
+///
+/// Note that we use indices into a `Dependencies` list to represent packages.
+/// This avoids excessive cloning.
 #[derive(Debug)]
 enum PossibleForks<'a> {
+    /// A group of dependencies (all with the same package name) where it is
+    /// known that no forks exist.
     NoForkPossible(Vec<usize>),
+    /// A group of groups dependencies (all with the same package name) where
+    /// it is possible for each group to correspond to a fork.
     PossiblyForking(PossibleForkGroups<'a>),
 }
 
@@ -2043,8 +2156,11 @@ impl<'a> PossibleForks<'a> {
     }
 }
 
+/// A list of groups of dependencies (all with the same package name), where
+/// each group may correspond to a fork.
 #[derive(Debug)]
 struct PossibleForkGroups<'a> {
+    /// The list of forks.
     forks: Vec<PossibleFork<'a>>,
 }
 
@@ -2062,6 +2178,20 @@ impl<'a> PossibleForkGroups<'a> {
     }
 }
 
+/// Intermediate state representing a single possible fork.
+///
+/// The key invariant here is that, beyond a singleton fork, for all packages
+/// in this fork, its marker expression must be overlapping with at least one
+/// other package's marker expression. That is, when considering whether a
+/// dependency specification with a conflicting package name provokes a fork
+/// or not, one must look at the existing possible groups of forks. If any of
+/// those groups have a package with an overlapping marker expression, then
+/// the conflicting package name cannot possibly introduce a new fork. But if
+/// there is no existing fork with an overlapping marker expression, then the
+/// conflict provokes a new fork.
+///
+/// As with other intermediate data types, we use indices into a list of
+/// `Dependencies` to represent packages to avoid excessive cloning.
 #[derive(Debug)]
 struct PossibleFork<'a> {
     packages: Vec<(usize, &'a MarkerTree)>,
@@ -2071,11 +2201,11 @@ impl<'a> PossibleFork<'a> {
     /// Returns true if and only if the given marker expression has a non-empty
     /// intersection with *any* of the package markers within this possible
     /// fork.
-    fn is_overlapping(&self, marker: &MarkerTree) -> bool {
+    fn is_overlapping(&self, candidate_package_markers: &MarkerTree) -> bool {
         use crate::marker::is_disjoint;
 
-        for (_, tree) in &self.packages {
-            if !is_disjoint(marker, tree) {
+        for (_, package_markers) in &self.packages {
+            if !is_disjoint(candidate_package_markers, package_markers) {
                 return true;
             }
         }
