@@ -5,30 +5,31 @@ use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tracing::debug;
 
-use distribution_types::{IndexLocations, Resolution};
-use install_wheel_rs::linker::LinkMode;
+use distribution_types::Resolution;
 use pep440_rs::Version;
 use uv_cache::Cache;
-use uv_client::{BaseClientBuilder, Connectivity, RegistryClientBuilder};
-use uv_configuration::{
-    Concurrency, ConfigSettings, ExtrasSpecification, NoBinary, NoBuild, PreviewMode, Reinstall,
-    SetupPyStrategy, Upgrade,
-};
+use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
+use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
 use uv_distribution::Workspace;
 use uv_fs::Simplified;
 use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
-use uv_resolver::{FlatIndex, InMemoryIndex, Options, RequiresPython};
-use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain, ToolchainRequest};
+use uv_resolver::{FlatIndex, InMemoryIndex, OptionsBuilder, RequiresPython};
+use uv_toolchain::{
+    Interpreter, PythonEnvironment, SystemPython, Toolchain, ToolchainRequest, VersionRequest,
+};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 use uv_warnings::warn_user;
 
 use crate::commands::pip;
 use crate::printer::Printer;
+use crate::settings::ResolverInstallerSettings;
 
+pub(crate) mod add;
 pub(crate) mod lock;
+pub(crate) mod remove;
 pub(crate) mod run;
 pub(crate) mod sync;
 
@@ -71,6 +72,22 @@ pub(crate) enum ProjectError {
     RequiresPython(#[from] uv_resolver::RequiresPythonError),
 }
 
+/// Compute the `Requires-Python` bound for the [`Workspace`].
+///
+/// For a [`Workspace`] with multiple packages, the `Requires-Python` bound is the union of the
+/// `Requires-Python` bounds of all the packages.
+pub(crate) fn find_requires_python(
+    workspace: &Workspace,
+) -> Result<Option<RequiresPython>, uv_resolver::RequiresPythonError> {
+    RequiresPython::union(workspace.packages().values().filter_map(|member| {
+        member
+            .pyproject_toml()
+            .project
+            .as_ref()
+            .and_then(|project| project.requires_python.as_ref())
+    }))
+}
+
 /// Find the virtual environment for the current project.
 pub(crate) fn find_environment(
     workspace: &Workspace,
@@ -79,64 +96,88 @@ pub(crate) fn find_environment(
     PythonEnvironment::from_root(workspace.venv(), cache)
 }
 
-/// Initialize a virtual environment for the current project.
-pub(crate) fn init_environment(
+/// Check if the given interpreter satisfies the project's requirements.
+pub(crate) fn interpreter_meets_requirements(
+    interpreter: &Interpreter,
+    requested_python: Option<&str>,
+    requires_python: Option<&RequiresPython>,
+    cache: &Cache,
+) -> bool {
+    // `--python` has highest precedence, after that we check `requires_python` from
+    // `pyproject.toml`. If `--python` and `requires_python` are mutually incompatible,
+    // we'll fail at the build or at last the install step when we aren't able to install
+    // the editable wheel for the current project into the venv.
+    // TODO(konsti): Do we want to support a workspace python version requirement?
+    if let Some(python) = requested_python {
+        let request = ToolchainRequest::parse(python);
+        if request.satisfied(interpreter, cache) {
+            debug!("Interpreter meets the requested python {}", request);
+            return true;
+        }
+
+        debug!("Interpreter does not meet the request {}", request);
+        return false;
+    };
+
+    if let Some(requires_python) = requires_python {
+        if requires_python.contains(interpreter.python_version()) {
+            debug!("Interpreter meets the project `Requires-Python` constraint {requires_python}");
+            return true;
+        }
+
+        debug!(
+            "Interpreter does not meet the project `Requires-Python` constraint {requires_python}"
+        );
+        return false;
+    };
+
+    // No requirement to check
+    true
+}
+
+/// Find the interpreter to use in the current project.
+pub(crate) fn find_interpreter(
     workspace: &Workspace,
     python: Option<&str>,
-    preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
-) -> Result<PythonEnvironment, ProjectError> {
-    let venv = workspace.root().join(".venv");
+) -> Result<Interpreter, ProjectError> {
+    let requires_python = find_requires_python(workspace)?;
 
-    let requires_python = workspace
-        .root_member()
-        .and_then(|root| root.project().requires_python.as_ref());
-
-    // Discover or create the virtual environment.
-    match PythonEnvironment::from_root(venv, cache) {
+    // Read from the virtual environment first
+    match find_environment(workspace, cache) {
         Ok(venv) => {
-            // `--python` has highest precedence, after that we check `requires_python` from
-            // `pyproject.toml`. If `--python` and `requires_python` are mutually incompatible,
-            // we'll fail at the build or at last the install step when we aren't able to install
-            // the editable wheel for the current project into the venv.
-            // TODO(konsti): Do we want to support a workspace python version requirement?
-            let is_satisfied = if let Some(python) = python {
-                ToolchainRequest::parse(python).satisfied(venv.interpreter(), cache)
-            } else if let Some(requires_python) = requires_python {
-                requires_python.contains(venv.interpreter().python_version())
-            } else {
-                true
-            };
-
-            if is_satisfied {
-                return Ok(venv);
+            if interpreter_meets_requirements(
+                venv.interpreter(),
+                python,
+                requires_python.as_ref(),
+                cache,
+            ) {
+                return Ok(venv.into_interpreter());
             }
-
-            writeln!(
-                printer.stderr(),
-                "Removing virtual environment at: {}",
-                venv.root().user_display().cyan()
-            )?;
-            fs_err::remove_dir_all(venv.root())
-                .context("Failed to remove existing virtual environment")?;
         }
         Err(uv_toolchain::Error::NotFound(_)) => {}
         Err(e) => return Err(e.into()),
-    }
+    };
 
-    // TODO(konsti): Extend `VersionRequest` to support `VersionSpecifiers`.
-    let requires_python_str = requires_python.map(ToString::to_string);
-    let interpreter = Toolchain::find(
-        python.or(requires_python_str.as_deref()),
-        // Otherwise we'll try to use the venv we just deleted.
-        SystemPython::Required,
-        preview,
-        cache,
-    )?
+    // Otherwise, find a system interpreter to use
+    let interpreter = if let Some(request) = python.map(ToolchainRequest::parse).or(requires_python
+        .as_ref()
+        .map(RequiresPython::specifiers)
+        .map(|specifiers| ToolchainRequest::Version(VersionRequest::Range(specifiers.clone()))))
+    {
+        Toolchain::find_requested(
+            &request,
+            SystemPython::Required,
+            PreviewMode::Enabled,
+            cache,
+        )
+    } else {
+        Toolchain::find(None, SystemPython::Required, PreviewMode::Enabled, cache)
+    }?
     .into_interpreter();
 
-    if let Some(requires_python) = requires_python {
+    if let Some(requires_python) = requires_python.as_ref() {
         if !requires_python.contains(interpreter.python_version()) {
             warn_user!(
                 "The Python {} you requested with {} is incompatible with the requirement of the \
@@ -155,6 +196,46 @@ pub(crate) fn init_environment(
         interpreter.sys_executable().user_display().cyan()
     )?;
 
+    Ok(interpreter)
+}
+
+/// Initialize a virtual environment for the current project.
+pub(crate) fn init_environment(
+    workspace: &Workspace,
+    python: Option<&str>,
+    cache: &Cache,
+    printer: Printer,
+) -> Result<PythonEnvironment, ProjectError> {
+    let requires_python = find_requires_python(workspace)?;
+
+    // Check if the environment exists and is sufficient
+    match find_environment(workspace, cache) {
+        Ok(venv) => {
+            if interpreter_meets_requirements(
+                venv.interpreter(),
+                python,
+                requires_python.as_ref(),
+                cache,
+            ) {
+                return Ok(venv);
+            }
+
+            // Remove the existing virtual environment if it doesn't meet the requirements
+            writeln!(
+                printer.stderr(),
+                "Removing virtual environment at: {}",
+                venv.root().user_display().cyan()
+            )?;
+            fs_err::remove_dir_all(venv.root())
+                .context("Failed to remove existing virtual environment")?;
+        }
+        Err(uv_toolchain::Error::NotFound(_)) => {}
+        Err(e) => return Err(e.into()),
+    };
+
+    // Find an interpreter to create the environment with
+    let interpreter = find_interpreter(workspace, python, cache, printer)?;
+
     let venv = workspace.venv();
     writeln!(
         printer.stderr(),
@@ -172,17 +253,38 @@ pub(crate) fn init_environment(
 }
 
 /// Update a [`PythonEnvironment`] to satisfy a set of [`RequirementsSource`]s.
+#[allow(clippy::too_many_arguments)]
 pub(crate) async fn update_environment(
     venv: PythonEnvironment,
     requirements: &[RequirementsSource],
-    index_locations: &IndexLocations,
+    settings: &ResolverInstallerSettings,
+    preview: PreviewMode,
     connectivity: Connectivity,
+    concurrency: Concurrency,
+    native_tls: bool,
     cache: &Cache,
     printer: Printer,
-    preview: PreviewMode,
 ) -> Result<PythonEnvironment> {
-    // TODO(zanieb): Support client configuration
-    let client_builder = BaseClientBuilder::default().connectivity(connectivity);
+    // Extract the project settings.
+    let ResolverInstallerSettings {
+        index_locations,
+        index_strategy,
+        keyring_provider,
+        resolution,
+        prerelease,
+        config_setting,
+        exclude_newer,
+        link_mode,
+        compile_bytecode,
+        upgrade,
+        reinstall,
+        build_options,
+    } = settings;
+
+    let client_builder = BaseClientBuilder::new()
+        .connectivity(connectivity)
+        .native_tls(native_tls)
+        .keyring(*keyring_provider);
 
     // Read all requirements from the provided sources.
     // TODO(zanieb): Consider allowing constraints and extras
@@ -220,35 +322,44 @@ pub(crate) async fn update_environment(
     let markers = venv.interpreter().markers();
 
     // Initialize the registry client.
-    // TODO(zanieb): Support client options e.g. offline, tls, etc.
     let client = RegistryClientBuilder::new(cache.clone())
+        .native_tls(native_tls)
         .connectivity(connectivity)
         .index_urls(index_locations.index_urls())
+        .index_strategy(*index_strategy)
+        .keyring(*keyring_provider)
         .markers(markers)
-        .platform(venv.interpreter().platform())
+        .platform(interpreter.platform())
         .build();
 
-    // TODO(charlie): Respect project configuration.
-    let build_isolation = BuildIsolation::default();
-    let compile = false;
-    let concurrency = Concurrency::default();
-    let config_settings = ConfigSettings::default();
-    let dry_run = false;
-    let extras = ExtrasSpecification::default();
-    let flat_index = FlatIndex::default();
+    let options = OptionsBuilder::new()
+        .resolution_mode(*resolution)
+        .prerelease_mode(*prerelease)
+        .exclude_newer(*exclude_newer)
+        .index_strategy(*index_strategy)
+        .build();
+
+    // Initialize any shared state.
     let git = GitResolver::default();
-    let dev = Vec::default();
-    let hasher = HashStrategy::default();
     let in_flight = InFlight::default();
     let index = InMemoryIndex::default();
-    let link_mode = LinkMode::default();
-    let no_binary = NoBinary::default();
-    let no_build = NoBuild::default();
-    let options = Options::default();
+
+    // TODO(charlie): These are all default values. We should consider whether we want to make them
+    // optional on the downstream APIs.
+    let build_isolation = BuildIsolation::default();
+    let dev = Vec::default();
+    let dry_run = false;
+    let extras = ExtrasSpecification::default();
+    let hasher = HashStrategy::default();
     let preferences = Vec::default();
-    let reinstall = Reinstall::default();
     let setup_py = SetupPyStrategy::default();
-    let upgrade = Upgrade::default();
+
+    // Resolve the flat indexes from `--find-links`.
+    let flat_index = {
+        let client = FlatIndexClient::new(&client, cache);
+        let entries = client.fetch(index_locations.flat_index()).await?;
+        FlatIndex::from_entries(entries, Some(tags), &hasher, build_options)
+    };
 
     // Create a build dispatch.
     let resolve_dispatch = BuildDispatch::new(
@@ -261,11 +372,10 @@ pub(crate) async fn update_environment(
         &git,
         &in_flight,
         setup_py,
-        &config_settings,
+        config_setting,
         build_isolation,
-        link_mode,
-        &no_build,
-        &no_binary,
+        *link_mode,
+        build_options,
         concurrency,
         preview,
     );
@@ -282,8 +392,8 @@ pub(crate) async fn update_environment(
         preferences,
         site_packages.clone(),
         &hasher,
-        &reinstall,
-        &upgrade,
+        reinstall,
+        upgrade,
         interpreter,
         Some(tags),
         Some(markers),
@@ -321,11 +431,10 @@ pub(crate) async fn update_environment(
             &git,
             &in_flight,
             setup_py,
-            &config_settings,
+            config_setting,
             build_isolation,
-            link_mode,
-            &no_build,
-            &no_binary,
+            *link_mode,
+            build_options,
             concurrency,
             preview,
         )
@@ -336,10 +445,10 @@ pub(crate) async fn update_environment(
         &resolution,
         site_packages,
         pip::operations::Modifications::Sufficient,
-        &reinstall,
-        &no_binary,
-        link_mode,
-        compile,
+        reinstall,
+        build_options,
+        *link_mode,
+        *compile_bytecode,
         index_locations,
         &hasher,
         tags,
