@@ -2,45 +2,43 @@ use std::ffi::OsString;
 use std::path::PathBuf;
 
 use anyhow::{Context, Result};
-use distribution_types::IndexLocations;
 use itertools::Itertools;
 use tokio::process::Command;
 use tracing::debug;
 
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity};
-use uv_configuration::{ExtrasSpecification, PreviewMode, Upgrade};
+use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode};
 use uv_distribution::{ProjectWorkspace, Workspace};
 use uv_normalize::PackageName;
 use uv_requirements::RequirementsSource;
-use uv_resolver::ExcludeNewer;
-use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain};
+use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain, ToolchainRequest};
 use uv_warnings::warn_user;
 
+use crate::commands::pip::operations::Modifications;
 use crate::commands::{project, ExitStatus};
 use crate::printer::Printer;
+use crate::settings::ResolverInstallerSettings;
 
 /// Run a command.
 #[allow(clippy::too_many_arguments)]
 pub(crate) async fn run(
-    index_locations: IndexLocations,
     extras: ExtrasSpecification,
     dev: bool,
     target: Option<String>,
     mut args: Vec<OsString>,
     requirements: Vec<RequirementsSource>,
     python: Option<String>,
-    upgrade: Upgrade,
-    exclude_newer: Option<ExcludeNewer>,
     package: Option<PackageName>,
+    settings: ResolverInstallerSettings,
     isolated: bool,
     preview: PreviewMode,
     connectivity: Connectivity,
+    concurrency: Concurrency,
+    native_tls: bool,
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
-    let client_builder = BaseClientBuilder::new().connectivity(connectivity);
-
     if preview.is_disabled() {
         warn_user!("`uv run` is experimental and may change without warning.");
     }
@@ -62,29 +60,27 @@ pub(crate) async fn run(
         } else {
             ProjectWorkspace::discover(&std::env::current_dir()?, None).await?
         };
-        let venv = project::init_environment(
-            project.workspace(),
-            python.as_deref(),
-            preview,
-            cache,
-            printer,
-        )?;
+        let venv =
+            project::init_environment(project.workspace(), python.as_deref(), cache, printer)?;
 
         // Lock and sync the environment.
-        let root_project_name = project
-            .current_project()
-            .pyproject_toml()
-            .project
-            .as_ref()
-            .map(|project| project.name.clone());
         let lock = project::lock::do_lock(
-            root_project_name,
             project.workspace(),
             venv.interpreter(),
-            &index_locations,
-            upgrade,
-            exclude_newer,
+            &settings.upgrade,
+            &settings.index_locations,
+            &settings.index_strategy,
+            &settings.keyring_provider,
+            &settings.resolution,
+            &settings.prerelease,
+            &settings.config_setting,
+            settings.exclude_newer.as_ref(),
+            &settings.link_mode,
+            &settings.build_options,
             preview,
+            connectivity,
+            concurrency,
+            native_tls,
             cache,
             printer,
         )
@@ -94,10 +90,21 @@ pub(crate) async fn run(
             project.workspace().root(),
             &venv,
             &lock,
-            &index_locations,
             extras,
             dev,
+            Modifications::Sufficient,
+            &settings.reinstall,
+            &settings.index_locations,
+            &settings.index_strategy,
+            &settings.keyring_provider,
+            &settings.config_setting,
+            &settings.link_mode,
+            &settings.compile_bytecode,
+            &settings.build_options,
             preview,
+            connectivity,
+            concurrency,
+            native_tls,
             cache,
             printer,
         )
@@ -113,13 +120,17 @@ pub(crate) async fn run(
     } else {
         debug!("Syncing ephemeral environment.");
 
+        let client_builder = BaseClientBuilder::new()
+            .connectivity(connectivity)
+            .native_tls(native_tls);
+
         // Discover an interpreter.
         let interpreter = if let Some(project_env) = &project_env {
             project_env.interpreter().clone()
         } else {
             // Note we force preview on during `uv run` for now since the entire interface is in preview
             Toolchain::find_or_fetch(
-                python.as_deref(),
+                python.as_deref().map(ToolchainRequest::parse),
                 SystemPython::Allowed,
                 PreviewMode::Enabled,
                 client_builder,
@@ -149,11 +160,13 @@ pub(crate) async fn run(
             project::update_environment(
                 venv,
                 &requirements,
-                &index_locations,
+                &settings,
+                preview,
                 connectivity,
+                concurrency,
+                native_tls,
                 cache,
                 printer,
-                preview,
             )
             .await?,
         )
