@@ -4,9 +4,10 @@
 use assert_cmd::assert::{Assert, OutputAssertExt};
 use assert_cmd::Command;
 use assert_fs::assert::PathAssert;
-use assert_fs::fixture::PathChild;
+use assert_fs::fixture::{ChildPath, PathChild};
 use regex::Regex;
 use std::borrow::BorrowMut;
+use std::collections::VecDeque;
 use std::env;
 use std::ffi::OsString;
 use std::iter::Iterator;
@@ -18,7 +19,7 @@ use uv_configuration::PreviewMode;
 use uv_cache::Cache;
 use uv_fs::Simplified;
 use uv_toolchain::managed::InstalledToolchains;
-use uv_toolchain::{PythonVersion, Toolchain};
+use uv_toolchain::{PythonVersion, Toolchain, ToolchainRequest};
 
 // Exclude any packages uploaded after this date.
 pub static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
@@ -26,7 +27,7 @@ pub static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
 /// Using a find links url allows using `--index-url` instead of `--extra-index-url` in tests
 /// to prevent dependency confusion attacks against our test suite.
 pub const BUILD_VENDOR_LINKS_URL: &str =
-    "https://raw.githubusercontent.com/astral-sh/packse/0.3.18/vendor/links.html";
+    "https://raw.githubusercontent.com/astral-sh/packse/0.3.27/vendor/links.html";
 
 #[doc(hidden)] // Macro and test context only, don't use directly.
 pub const INSTA_FILTERS: &[(&str, &str)] = &[
@@ -35,6 +36,9 @@ pub const INSTA_FILTERS: &[(&str, &str)] = &[
     (r"(\s|\()(\d+m )?(\d+\.)?\d+(ms|s)", "$1[TIME]"),
     // File sizes
     (r"(\s|\()(\d+\.)?\d+([KM]i)?B", "$1[SIZE]"),
+    // Timestamps
+    (r"tv_sec: \d+", "tv_sec: [TIME]"),
+    (r"tv_nsec: \d+", "tv_nsec: [TIME]"),
     // Rewrite Windows output to Unix output
     (r"\\([\w\d])", "/$1"),
     (r"uv.exe", "uv"),
@@ -59,15 +63,19 @@ pub struct TestContext {
     pub python_version: String,
     pub workspace_root: PathBuf,
 
+    // Additional Python versions
+    python_versions: Vec<(PythonVersion, PathBuf)>,
+
     // Standard filters for this test context
-    filters: Vec<(String, String)>,
+    filters: VecDeque<(String, String)>,
 }
 
 impl TestContext {
     pub fn new(python_version: &str) -> Self {
         let temp_dir = assert_fs::TempDir::new().expect("Failed to create temp dir");
         let cache_dir = assert_fs::TempDir::new().expect("Failed to create cache dir");
-        let venv = create_venv(&temp_dir, &cache_dir, python_version);
+        let python = get_toolchain(python_version);
+        let venv = create_venv_from_executable(&temp_dir, &cache_dir, &python);
 
         // The workspace root directory is not available without walking up the tree
         // https://github.com/rust-lang/cargo/issues/3946
@@ -83,7 +91,8 @@ impl TestContext {
         let python_version =
             PythonVersion::from_str(python_version).expect("Tests must use valid Python versions");
 
-        let mut filters = Vec::new();
+        let mut filters = VecDeque::new();
+
         filters.extend(
             Self::path_patterns(&cache_dir)
                 .into_iter()
@@ -111,7 +120,7 @@ impl TestContext {
         );
 
         // Account for [`Simplified::user_display`] which is relative to the command working directory
-        filters.push((
+        filters.push_back((
             Self::path_pattern(
                 site_packages
                     .strip_prefix(&temp_dir)
@@ -119,7 +128,7 @@ impl TestContext {
             ),
             "[SITE_PACKAGES]/".to_string(),
         ));
-        filters.push((
+        filters.push_back((
             Self::path_pattern(
                 venv.strip_prefix(&temp_dir)
                     .expect("The test virtual environment directory is always in the tempdir"),
@@ -129,33 +138,67 @@ impl TestContext {
 
         // Filter non-deterministic temporary directory names
         // Note we apply this _after_ all the full paths to avoid breaking their matching
-        filters.push((r"(\\|\/)\.tmp.*(\\|\/)".to_string(), "/[TMP]/".to_string()));
+        filters.push_back((r"(\\|\/)\.tmp.*(\\|\/)".to_string(), "/[TMP]/".to_string()));
 
         // Account for platform prefix differences `file://` (Unix) vs `file:///` (Windows)
-        filters.push((r"file:///".to_string(), "file://".to_string()));
+        filters.push_back((r"file:///".to_string(), "file://".to_string()));
 
         // Destroy any remaining UNC prefixes (Windows only)
-        filters.push((r"\\\\\?\\".to_string(), String::new()));
+        filters.push_back((r"\\\\\?\\".to_string(), String::new()));
 
-        // Add Python patch version filtering unless explicitly requested to ensure
-        // snapshots are patch version agnostic when it is not a part of the test.
-        if python_version.patch().is_none() {
-            filters.push((
-                format!(
-                    r"({})\.\d+",
-                    regex::escape(python_version.to_string().as_str())
-                ),
-                "$1.[X]".to_string(),
-            ));
-        }
-
-        Self {
+        let mut result = Self {
             temp_dir,
             cache_dir,
             venv,
             python_version: python_version.to_string(),
             filters,
+            python_versions: Vec::new(),
             workspace_root,
+        };
+
+        result.add_filters_for_python_version(&python_version, python);
+
+        result
+    }
+
+    pub fn new_with_versions(python_versions: &[&str]) -> Self {
+        let mut context = Self::new(
+            python_versions
+                .first()
+                .expect("At least one test Python version must be provided"),
+        );
+
+        let python_versions: Vec<_> = python_versions
+            .iter()
+            .map(|version| PythonVersion::from_str(version).unwrap())
+            .zip(
+                python_toolchains_for_versions(&context.temp_dir, python_versions)
+                    .expect("Failed to find test Python versions"),
+            )
+            .collect();
+
+        for (version, path) in &python_versions {
+            context.add_filters_for_python_version(version, path.clone());
+        }
+
+        context.python_versions = python_versions;
+
+        context
+    }
+
+    fn add_filters_for_python_version(&mut self, version: &PythonVersion, executable: PathBuf) {
+        // Add filtering for the interpreter path
+        for pattern in Self::path_patterns(executable) {
+            self.filters
+                .push_front((format!("{pattern}python.*"), format!("[PYTHON-{version}]")));
+        }
+        // Add Python patch version filtering unless explicitly requested to ensure
+        // snapshots are patch version agnostic when it is not a part of the test.
+        if version.patch().is_none() {
+            self.filters.push_back((
+                format!(r"({})\.\d+", regex::escape(version.to_string().as_str())),
+                "$1.[X]".to_string(),
+            ));
         }
     }
 
@@ -180,7 +223,7 @@ impl TestContext {
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
             .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .current_dir(self.temp_dir.path());
 
         if cfg!(all(windows, debug_assertions)) {
@@ -214,7 +257,7 @@ impl TestContext {
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
             .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .current_dir(&self.temp_dir);
 
         if cfg!(all(windows, debug_assertions)) {
@@ -234,9 +277,9 @@ impl TestContext {
             .arg("--cache-dir")
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .current_dir(&self.temp_dir);
 
         if cfg!(all(windows, debug_assertions)) {
@@ -254,7 +297,7 @@ impl TestContext {
         command
             .arg("--exclude-newer")
             .arg(EXCLUDE_NEWER)
-            .env("UV_TEST_PYTHON_PATH", "/dev/null");
+            .env("UV_TEST_PYTHON_PATH", &self.python_path());
         command
     }
 
@@ -272,7 +315,35 @@ impl TestContext {
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
             .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
+            .current_dir(&self.temp_dir);
+
+        if cfg!(all(windows, debug_assertions)) {
+            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
+            // default windows stack of 1MB
+            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
+        }
+
+        command
+    }
+
+    pub fn toolchains_dir(&self) -> ChildPath {
+        self.temp_dir.child("toolchains")
+    }
+
+    /// Create a `uv toolchain find` command with options shared across scenarios.
+    pub fn toolchain_find(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command
+            .arg("toolchain")
+            .arg("find")
+            .arg("--cache-dir")
+            .arg(self.cache_dir.path())
+            .env("VIRTUAL_ENV", self.venv.as_os_str())
+            .env("UV_NO_WRAP", "1")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
+            .env("UV_PREVIEW", "1")
+            .env("UV_TOOLCHAIN_DIR", self.toolchains_dir().as_os_str())
             .current_dir(&self.temp_dir);
 
         if cfg!(all(windows, debug_assertions)) {
@@ -305,7 +376,7 @@ impl TestContext {
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
             .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", "/dev/null")
+            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .current_dir(&self.temp_dir);
 
         if cfg!(all(windows, debug_assertions)) {
@@ -426,6 +497,10 @@ impl TestContext {
         )
     }
 
+    fn python_path(&self) -> OsString {
+        std::env::join_paths(self.python_versions.iter().map(|(_, path)| path)).unwrap()
+    }
+
     /// Standard snapshot filters _plus_ those for this test context.
     pub fn filters(&self) -> Vec<(&str, &str)> {
         // Put test context snapshots before the default filters
@@ -450,6 +525,31 @@ impl TestContext {
             &format!("{}{}", self.python_kind(), self.python_version),
         )
     }
+
+    /// Reset the virtual environment in the test context.
+    pub fn reset_venv(&self) {
+        create_venv_from_executable(
+            &self.temp_dir,
+            &self.cache_dir,
+            &get_toolchain(&self.python_version),
+        );
+    }
+
+    /// Create a new virtual environment named `.venv` in the test context.
+    fn create_venv(&mut self, python: &str) -> PathBuf {
+        let parent = self.temp_dir.to_path_buf();
+        self.create_venv_in_parent(parent, python)
+    }
+
+    /// Create a new virtual environment named `.venv` in the given directory.
+    fn create_venv_in_parent<P: AsRef<Path>>(&mut self, path: P, python: &str) -> PathBuf {
+        let executable = get_toolchain(python);
+        self.add_filters_for_python_version(
+            &PythonVersion::from_str(python).unwrap(),
+            executable.clone(),
+        );
+        create_venv_from_executable(&ChildPath::new(path.as_ref()), &self.cache_dir, &executable)
+    }
 }
 
 pub fn site_packages_path(venv: &Path, python: &str) -> PathBuf {
@@ -503,14 +603,14 @@ pub fn get_toolchain(python: &str) -> PathBuf {
 }
 
 /// Create a virtual environment named `.venv` in a temporary directory with the given
-/// Python version. Expected format for `python` is "<version>".
-pub fn create_venv<Parent: assert_fs::prelude::PathChild + AsRef<std::path::Path>>(
+/// Python executable.
+pub fn create_venv_from_executable<
+    Parent: assert_fs::prelude::PathChild + AsRef<std::path::Path>,
+>(
     temp_dir: &Parent,
     cache_dir: &assert_fs::TempDir,
-    python: &str,
+    python: &Path,
 ) -> PathBuf {
-    let python = get_toolchain(python);
-
     let venv = temp_dir.child(".venv");
     Command::new(get_bin())
         .arg("venv")
@@ -540,6 +640,19 @@ pub fn python_path_with_versions(
     temp_dir: &assert_fs::TempDir,
     python_versions: &[&str],
 ) -> anyhow::Result<OsString> {
+    Ok(std::env::join_paths(python_toolchains_for_versions(
+        temp_dir,
+        python_versions,
+    )?)?)
+}
+
+/// Create a `PATH` with the requested Python versions available in order.
+///
+/// Generally this should be used with `UV_TEST_PYTHON_PATH`.
+pub fn python_toolchains_for_versions(
+    temp_dir: &assert_fs::TempDir,
+    python_versions: &[&str],
+) -> anyhow::Result<Vec<PathBuf>> {
     let cache = Cache::from_path(temp_dir.child("cache").to_path_buf()).init()?;
     let selected_pythons = python_versions
         .iter()
@@ -565,7 +678,7 @@ pub fn python_path_with_versions(
             if inner.is_empty() {
                 // Fallback to a system lookup if we failed to find one in the toolchain directory
                 if let Ok(toolchain) = Toolchain::find(
-                    Some(python_version),
+                    Some(ToolchainRequest::parse(python_version)),
                     // Without required, we could pick the current venv here and the test fails
                     // because the venv subcommand requires a system interpreter.
                     uv_toolchain::SystemPython::Required,
@@ -592,7 +705,7 @@ pub fn python_path_with_versions(
         "Failed to fulfill requested test Python versions: {selected_pythons:?}"
     );
 
-    Ok(env::join_paths(selected_pythons)?)
+    Ok(selected_pythons)
 }
 
 #[derive(Debug, Copy, Clone)]
